<?php

/**
 * Module de remontée de piratage
 *
 * Ce module détecte les requêtes malveillantes et les remonte
 * automatiquement à un endpoint centralisé.
 *
 * @author Maël Donnart <mael.donnart@nwb.fr>
 * @version 0.1.3
 */

// File d'attente en mémoire pour les tentatives échouées
$GLOBALS['cybersec_retry_queue'] = [];

// Cache des patterns chargés depuis le fichier JSON
$GLOBALS['cybersec_patterns_cache'] = null;

/**
 * Charge les patterns de menaces depuis un fichier JSON
 *
 * @param string $jsonFilePath Chemin vers le fichier JSON des patterns
 * @return array Tableau des patterns ou tableau vide en cas d'erreur
 * @throws Exception Si le fichier n'existe pas ou est invalide
 */
function loadThreatPatterns($jsonFilePath)
{
    // Utiliser le cache si déjà chargé
    if ($GLOBALS['cybersec_patterns_cache'] !== null) {
        return $GLOBALS['cybersec_patterns_cache'];
    }

    // Vérifier que le fichier existe
    if (!file_exists($jsonFilePath)) {
        throw new Exception("Fichier de patterns non trouvé: " . $jsonFilePath . ". Veuillez consulter la documentation pour configurer correctement les patterns de menaces.");
    }

    // Lire le contenu du fichier
    $jsonContent = file_get_contents($jsonFilePath);
    if ($jsonContent === false) {
        throw new Exception("Impossible de lire le fichier de patterns: " . $jsonFilePath . ". Vérifiez les permissions du fichier.");
    }

    // Décoder le JSON
    $patternsData = json_decode($jsonContent, true);
    if ($patternsData === null) {
        throw new Exception("JSON invalide dans le fichier de patterns: " . $jsonFilePath . ". Veuillez vérifier la syntaxe JSON du fichier.");
    }

    // Extraire les patterns du format JSON
    $patterns = [];
    if (isset($patternsData['threat_patterns']) && is_array($patternsData['threat_patterns'])) {
        $patterns = $patternsData['threat_patterns'];
    }

    // Mettre en cache pour les prochains appels
    $GLOBALS['cybersec_patterns_cache'] = $patterns;

    return $patterns;
}

/**
 * Signale une tentative de piratage détectée
 *
 * @param array $config Configuration optionnelle pour surcharger les paramètres
 * @param array $projectPatterns Patterns de détection - OBLIGATOIRE pour la détection automatique
 * @return bool True si l'envoi a réussi, false sinon
 * @throws Exception Si la configuration est invalide
 */
function signalHackingAttempt($config = [], $projectPatterns = [])
{
    // Configuration par défaut
    $defaultConfig = [
        'endpoint' => 'https://n8n.kub.nwb.fr/webhook/signalHackingAttempt',
        'auth_token' => null,
        'timeout' => 2,
        'max_retries' => 3,
        'retry_delays' => [1, 2, 4] // Délais en secondes pour les retry
    ];

    // Si aucun pattern n'est trouvé, on ne fait rien
    if (empty($projectPatterns) || !is_array($projectPatterns)) {
        throw new Exception("Aucun pattern de menace fourni. Veuillez consulter la documentation pour configurer correctement les patterns de détection.");
    }

    // Fusion avec la configuration fournie
    $finalConfig = array_merge($defaultConfig, $config);

    // Si l'endpoint n'est pas défini par défaut, on utilise la variable d'environnement CYBERSECMODULE_URL
    if (empty($finalConfig['endpoint']) && !empty(getenv('CYBERSECMODULE_URL'))) {
        $finalConfig['endpoint'] = getenv('CYBERSECMODULE_URL');
    }
    // Si l'auth_token n'est pas défini par défaut, on utilise la variable d'environnement CYBERSECMODULE_TOKEN
    if (empty($finalConfig['auth_token'])) {
        $finalConfig['auth_token'] = getenv('CYBERSECMODULE_TOKEN');
    }

    // Vérifier que le token d'authentification est disponible
    if (empty($finalConfig['auth_token'])) {
        throw new Exception("Token d'authentification manquant. Veuillez configurer la variable d'environnement CYBERSECMODULE_TOKEN ou passer le token en paramètre.");
    }

    // Récupération de l'URI de la requête
    $requestUri = isset($_SERVER['REQUEST_URI']) ? $_SERVER['REQUEST_URI'] : '';

    // Si l'URI est vide, on ne fait rien
    if (empty($requestUri)) {
        throw new Exception("URI de la requête vide. Le module CyberSec ne peut fonctionner que dans un contexte web avec REQUEST_URI.");
    }

    $isPatternMatch = false;
    $detectedPatternInfo = null;

    // Vérification si l'URI correspond à un pattern
    foreach ($projectPatterns as $patternInfo) {
        if (is_array($patternInfo) && isset($patternInfo['threat_patch'])) {
            $threatPatch = $patternInfo['threat_patch'];
            $patternDescription = isset($patternInfo['pattern']) ? $patternInfo['pattern'] : 'pattern non défini';
        }

        // Vérifier si l'URI contient le threat_patch
        if (strpos($requestUri, $threatPatch) !== false) {
            $isPatternMatch = true;
            $detectedPatternInfo = [
                'threat_patch' => $threatPatch,
                'pattern' => $patternDescription
            ];
            break;
        }
    }

    // Si aucun pattern ne correspond, pas de rapport à envoyer
    if (!$isPatternMatch) {
        return false;
    }

    // MENACE DÉTECTÉE : Un pattern correspond, on envoie le rapport
    // Collecte des données avec la description du pattern
    $data = _cybersec_collectData($detectedPatternInfo);

    // Tentative d'envoi avec retry
    $success = _cybersec_sendWithRetry($data, $finalConfig);

    return $success;
}

/**
 * Collecte les données des superglobales et forme le payload
 *
 * @param array $patternInfo Informations sur le pattern détecté avec les clés 'pattern' et 'threat_patch'
 * @return array Le payload à envoyer
 */
function _cybersec_collectData($patternInfo)
{
    // Extraction de l'IP
    $ip = _cybersec_getClientIp();

    // Timestamp ISO8601
    $timestamp = date('c');

    // Collecte des données des superglobales
    $data = [
        'get' => $_GET,
        'post' => $_POST,
        'server' => $_SERVER,
        'cookie' => $_COOKIE
    ];

    // Hachage des données sensibles
    $data = _cybersec_hashSensitiveData($data);

    // Formation du payload final avec format combiné : "description - threat_patch"
    $payload = [
        'ip' => $ip,
        'timestamp' => $timestamp,
        'pattern' => $patternInfo['pattern'] . ' - ' . $patternInfo['threat_patch'],
        'data' => $data
    ];

    return $payload;
}

/**
 * Obtient l'adresse IP du client
 *
 * @return string L'adresse IP du client
 */
function _cybersec_getClientIp()
{
    $ipKeys = [
        'HTTP_CF_CONNECTING_IP',     // Cloudflare
        'HTTP_X_FORWARDED_FOR',      // Proxy
        'HTTP_X_FORWARDED',          // Proxy
        'HTTP_X_CLUSTER_CLIENT_IP',  // Cluster
        'HTTP_FORWARDED_FOR',        // Proxy
        'HTTP_FORWARDED',            // Proxy
        'REMOTE_ADDR'                // Standard
    ];

    foreach ($ipKeys as $key) {
        if (!empty($_SERVER[$key])) {
            $ip = $_SERVER[$key];
            // Si c'est une liste d'IPs, prendre la première
            if (strpos($ip, ',') !== false) {
                $ip = trim(explode(',', $ip)[0]);
            }
            // Validation basique de l'IP
            if (filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_NO_PRIV_RANGE | FILTER_FLAG_NO_RES_RANGE)) {
                return $ip;
            }
        }
    }

    return isset($_SERVER['REMOTE_ADDR']) ? $_SERVER['REMOTE_ADDR'] : '127.0.0.1';
}

/**
 * Hache les données sensibles avec SHA-256
 *
 * @param array $data Les données à traiter
 * @return array Les données avec les clés sensibles hachées
 */
function _cybersec_hashSensitiveData($data)
{
    $sensitiveKeys = [
        'HTTP_AUTHORIZATION',
        'CYBERSECMODULE_TOKEN',
        'PHP_AUTH_PW',
        'PHP_AUTH_USER'
    ];

    // Hachage dans $_SERVER
    if (isset($data['server'])) {
        foreach ($sensitiveKeys as $key) {
            if (isset($data['server'][$key])) {
                $data['server'][$key] = hash('sha256', $data['server'][$key]);
            }
        }
    }

    // Hachage des cookies de session
    if (isset($data['cookie'])) {
        foreach ($data['cookie'] as $key => $value) {
            if (
                strpos(strtolower($key), 'sess') !== false ||
                strpos(strtolower($key), 'phpsessid') !== false
            ) {
                $data['cookie'][$key] = hash('sha256', $value);
            }
        }
    }

    return $data;
}

/**
 * Envoie les données avec système de retry
 *
 * @param array $data Les données à envoyer
 * @param array $config La configuration
 * @return bool True si succès, false sinon
 */
function _cybersec_sendWithRetry($data, $config)
{
    $maxRetries = $config['max_retries'];
    $retryDelays = $config['retry_delays'];

    for ($attempt = 0; $attempt < $maxRetries; $attempt++) {
        $success = _cybersec_sendHttpRequest($data, $config);

        if ($success) {
            return true;
        }

        // Si ce n'est pas la dernière tentative, attendre avant le retry
        if ($attempt < $maxRetries - 1) {
            $delay = isset($retryDelays[$attempt]) ? $retryDelays[$attempt] : 1;
            sleep($delay);
        }
    }

    // Échec après tous les retries - ajouter à la file d'attente en mémoire
    _cybersec_addToRetryQueue($data, $config);

    return false;
}

/**
 * Envoie une requête HTTP POST
 *
 * @param array $data Les données à envoyer
 * @param array $config La configuration
 * @return bool True si succès (200), false sinon
 */
function _cybersec_sendHttpRequest($data, $config)
{
    $endpoint = $config['endpoint'];
    $authToken = $config['auth_token'];
    $timeout = $config['timeout'];

    // Préparation du payload JSON
    $jsonData = json_encode($data);

    // Compression GZIP optionnelle
    $compressed = false;
    if (function_exists('gzencode')) {
        $compressedData = gzencode($jsonData);
        if ($compressedData && strlen($compressedData) < strlen($jsonData)) {
            $jsonData = $compressedData;
            $compressed = true;
        }
    }

    // Préparation des en-têtes
    $headers = [
        'Content-Type: application/json',
        'Content-Length: ' . strlen($jsonData)
    ];

    if ($compressed) {
        $headers[] = 'Content-Encoding: gzip';
    }

    if (!empty($authToken)) {
        $headers[] = 'Authorization: Bearer ' . $authToken;
    }

    // Utilisation de cURL si disponible, sinon file_get_contents
    if (function_exists('curl_init')) {
        return _cybersec_sendWithCurl($endpoint, $jsonData, $headers, $timeout);
    } else {
        return _cybersec_sendWithFileGetContents($endpoint, $jsonData, $headers, $timeout);
    }
}

/**
 * Envoie via cURL
 *
 * @param string $endpoint L'URL de destination
 * @param string $jsonData Les données JSON
 * @param array $headers Les en-têtes HTTP
 * @param int $timeout Le timeout en secondes
 * @return bool True si succès, false sinon
 */
function _cybersec_sendWithCurl($endpoint, $jsonData, $headers, $timeout)
{
    $ch = curl_init();

    curl_setopt_array($ch, [
        CURLOPT_URL => $endpoint,
        CURLOPT_POST => true,
        CURLOPT_POSTFIELDS => $jsonData,
        CURLOPT_HTTPHEADER => $headers,
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_TIMEOUT => $timeout,
        CURLOPT_FOLLOWLOCATION => false,
        CURLOPT_SSL_VERIFYPEER => true,
        CURLOPT_SSL_VERIFYHOST => 2
    ]);

    $response = curl_exec($ch);
    $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);

    curl_close($ch);

    return $httpCode === 200;
}

/**
 * Envoie via file_get_contents
 *
 * @param string $endpoint L'URL de destination
 * @param string $jsonData Les données JSON
 * @param array $headers Les en-têtes HTTP
 * @param int $timeout Le timeout en secondes
 * @return bool True si succès, false sinon
 */
function _cybersec_sendWithFileGetContents($endpoint, $jsonData, $headers, $timeout)
{
    $context = stream_context_create([
        'http' => [
            'method' => 'POST',
            'header' => implode("\r\n", $headers),
            'content' => $jsonData,
            'timeout' => $timeout,
            'ignore_errors' => true
        ]
    ]);

    $response = file_get_contents($endpoint, false, $context);

    // Vérification du code de réponse
    if (isset($http_response_header)) {
        $statusLine = $http_response_header[0];
        return strpos($statusLine, '200') !== false;
    }

    return false;
}

/**
 * Ajoute une tentative échouée à la file d'attente en mémoire
 *
 * @param array $data Les données
 * @param array $config La configuration
 */
function _cybersec_addToRetryQueue($data, $config)
{
    $GLOBALS['cybersec_retry_queue'][] = [
        'data' => $data,
        'config' => $config,
        'timestamp' => time()
    ];
}

/**
 * Traite la file d'attente des tentatives échouées
 * Cette fonction est destinée à être appelée par un worker CRON
 *
 * @return int Nombre de tentatives traitées avec succès
 */
function processCyberSecRetryQueue()
{
    if (empty($GLOBALS['cybersec_retry_queue'])) {
        return 0;
    }

    $processedCount = 0;
    $remainingQueue = [];

    foreach ($GLOBALS['cybersec_retry_queue'] as $item) {
        $success = _cybersec_sendHttpRequest($item['data'], $item['config']);

        if ($success) {
            $processedCount++;
        } else {
            // Garder en file d'attente si l'échec persiste
            $remainingQueue[] = $item;
        }
    }

    // Mettre à jour la file d'attente
    $GLOBALS['cybersec_retry_queue'] = $remainingQueue;

    return $processedCount;
}
